﻿//----------------------------------------------------
// CreateStructure.cs (c) 2019 by Enes Siljak
//----------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Interop;
using System.IO;
using System.Text;
using Autodesk.AdvanceSteel.CADAccess;
using Autodesk.AdvanceSteel.CADLink.Database;
using Autodesk.AdvanceSteel.ConstructionHelper;
using Autodesk.AdvanceSteel.ConstructionTypes;
using Autodesk.AdvanceSteel.DocumentManagement;
using Autodesk.AdvanceSteel.Geometry;
using Autodesk.AdvanceSteel.Modelling;
using Autodesk.AdvanceSteel.Profiles;
using Autodesk.AdvanceSteel.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using ASObjectId = Autodesk.AdvanceSteel.CADLink.Database.ObjectId;

namespace AsCreateStructure
{
    public class CreateStructure
    {
        enum BLOCKDEF
        {
            STEELSECTION,
            POINTS,
            LINES,
            ADDTOLINES
        }

        enum STEELSECPARAMS
        {
            TYPE,
            ID,
            MATERIAL,
            LABEL,      
            Y_OFFSET,
            Z_OFFSET,
            INSERTION_POINT
        }

        enum ADDTOLINESPARAMS
        {
            SELECT_LINES,
            SET_PROPERTY,
            SET_THETA
        }

        char[] null_delimiter = new char[0];
        char[] block_delimiter = new char[] { '#' };
        char[] prop_delimiter = new char[] { ';' };
        char[] whitesp_delimiter = new char[] { ' ' };

        List<SteelSection> sections = new List<SteelSection>();
        List<Point3d> points = new List<Point3d>();
        List<BeamProperties> beamprops = new List<BeamProperties>();

        [CommandMethodAttribute("XEN_GROUP", "XEN_CREATE_STRUCTURE", "XEN_CREATE_STRUCTURE", CommandFlags.Modal)]
        public void CreateStruct()
        {
            sections.Clear();
            points.Clear();
            beamprops.Clear();
            
            Autodesk.AutoCAD.Windows.OpenFileDialog ofd =
                new Autodesk.AutoCAD.Windows.OpenFileDialog("Open Text File", 
                null,
                //"C:\\Users\\Admin\\Documents\\Visual Studio 2017\\Projects\\AsCreateStructure", 
                "txt", "Open file",
                //Autodesk.AutoCAD.Windows.OpenFileDialog.OpenFileDialogFlags.DefaultIsFolder |
                Autodesk.AutoCAD.Windows.OpenFileDialog.OpenFileDialogFlags.ForceDefaultFolder);

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                if (ReadInputFile(ofd.Filename))
                {
                    Make();
                }
            }
        }

        private void Make()
        {
            DocumentManager.LockCurrentDocument();
            Autodesk.AdvanceSteel.CADAccess.Transaction tr = Autodesk.AdvanceSteel.CADAccess.TransactionManager.StartTransaction();

            try
            {
                for (int i = 0; i < beamprops.Count; i++)
                {
                    BeamProperties bp = beamprops[i];
                    SteelSection sec = bp.steelSection;
                          
                    StraightBeam myBeam = new StraightBeam(sec.getProfileName(), bp.start, bp.end, Vector3d.kZAxis);
                    myBeam.Material = sec.getMaterial();
                    myBeam.Angle = bp.theta_deg;
                    /*
            *alignement of beam to be finished 
            */

                    myBeam.WriteToDb();         
                }

            }
            catch (System.Exception ex)
            {
                System.Diagnostics.Debug.Assert(false, ex.Message);
            }
            tr.Commit();
            DocumentManager.UnlockCurrentDocument();
            
        }

        private bool ReadInputFile(string Filename)
        {
            if (!File.Exists(Filename))
            {
                return false;
            }
            else
            { 
                string readText = File.ReadAllText(Filename);
                string[] block_defs = readText.Split(block_delimiter, StringSplitOptions.RemoveEmptyEntries);

                for (int i = 0; i < block_defs.Length; i += 2)
                {
                    try
                    {
                        BLOCKDEF bldef = block_defs[i].ToEnum<BLOCKDEF>();

                        switch (bldef)
                        {
                            case BLOCKDEF.STEELSECTION:
                                if (!ReadSection(block_defs[i + 1])) return false;
                                break;
                            case BLOCKDEF.POINTS:
                                if (!ReadPoints(block_defs[i + 1])) return false;
                                break;
                            case BLOCKDEF.LINES:
                                if (!ReadLines(block_defs[i + 1])) return false;
                                break;
                            case BLOCKDEF.ADDTOLINES:
                                if (!ReadAdditions(block_defs[i + 1])) return false;
                                break;
                        }

                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Error at: " + block_defs[i] + "\n" + ex.Message);
                        return false;
                    }
                }
                //printInput();
                return true;
            }
        }

        private bool ReadSection(string block) { 
            try
            {
                string label = "Default Name 1";
                STEELSECTYPE type = 0;
                string secIndex = "", material = "";
                INSERTION_POINT insertionpoint = INSERTION_POINT.CENTROID;
                double y_offset = 0, z_offset = 0;

                string[] properties = block.Split(prop_delimiter, StringSplitOptions.RemoveEmptyEntries);
                string[] key_val;
                string ptr;
                
                for (int i = 0; i < properties.Length; i++)
                {
                    ptr = properties[i].Trim();
                    if (String.IsNullOrEmpty(ptr)) continue;
                    
                    key_val = ptr.Split(whitesp_delimiter, 2);
                    key_val[1] = key_val[1].TrimStart();
                    
                    STEELSECPARAMS sp = key_val[0].ToEnum<STEELSECPARAMS>();
                    
                    switch (sp)
                    {
                        case STEELSECPARAMS.LABEL:
                            label = key_val[1];
                            break;
                        case STEELSECPARAMS.TYPE:
                            type = key_val[1].ToEnum<STEELSECTYPE>();
                            break;
                        case STEELSECPARAMS.ID:
                            secIndex = key_val[1];
                            break;
                        case STEELSECPARAMS.MATERIAL:
                            material = key_val[1];
                            break;
                        case STEELSECPARAMS.INSERTION_POINT:
                            insertionpoint = key_val[1].ToEnum<INSERTION_POINT>();
                            break;
                            
                        case STEELSECPARAMS.Y_OFFSET:
                            y_offset = Double.Parse(key_val[1]);
                            break;
                        case STEELSECPARAMS.Z_OFFSET:
                            z_offset = Double.Parse(key_val[1]);
                            break; 
                    } 
                }

                
                SteelSection steel_sec = new SteelSection(type, secIndex, material, 
                    insertionpoint, y_offset, z_offset, label);
                sections.Add(steel_sec);

                return true;

            }catch(Exception ex)
            {
                Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
                ed.WriteMessage(ex.Message);
                return false;
            }
            
        }

        private bool ReadPoints(string block)
        {
            string[] ptstr = block.Split(prop_delimiter, StringSplitOptions.RemoveEmptyEntries);
            string[] pts;
            string ptln;

            try
            {
                for (int i = 0; i < ptstr.Length; i++)
                {
                    ptln = ptstr[i].Trim();
                    if (String.IsNullOrEmpty(ptln)) continue;
                    pts = ptln.Split();
                    Point3d pt = new Point3d(Double.Parse(pts[0]), Double.Parse(pts[1]), Double.Parse(pts[2]));

                    points.Add(pt);
                }

                return true;

            }catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }

        }

        private bool ReadLines(string block)
        {
            string[] str = block.Split(prop_delimiter, StringSplitOptions.RemoveEmptyEntries);
            string[] lns;
            string[] lnnds;
            string lnln;

            try
            {
                for (int i = 0; i < str.Length; i++)
                {
                    lnln = str[i].Trim();
                    if (String.IsNullOrEmpty(lnln)) continue;
                    lnnds = lnln.Split(',');
                    lns = lnnds[0].Split();

                    Point3d beamstart = points[int.Parse(lns[0])];
                    Point3d beamend = points[int.Parse(lns[1])];

                    BeamProperties bp = new BeamProperties(beamstart, beamend);

                    beamprops.Add(bp);
                }

                return true;

            }catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }
        }

        private bool ReadAdditions(string block)
        {
            bool allLines = false;
            int[] selectedLn = null;

            string[] inputLines = block.Split(prop_delimiter, StringSplitOptions.RemoveEmptyEntries);
            string[] key_val;
            string inputLn;
            string[] sn;

            try
            {
                for (int i = 0; i < inputLines.Length; i++)
                {
                    inputLn = inputLines[i].Trim();
                    if (String.IsNullOrEmpty(inputLn)) continue;
                    key_val = inputLn.Split(whitesp_delimiter, 2);
                    key_val[1] = key_val[1].Trim();

                    ADDTOLINESPARAMS key = key_val[0].ToEnum<ADDTOLINESPARAMS>();

                    switch (key)
                    {
                        case ADDTOLINESPARAMS.SELECT_LINES:
                            if (String.Equals(key_val[1], "ALL", StringComparison.OrdinalIgnoreCase))
                            {
                                selectedLn = null;
                                allLines = true;

                            }
                            else if (key_val[1].IndexOf(":") > 0)
                            {
                                sn = key_val[1].Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);
                                int startIdx = int.Parse(sn[0]);
                                int endIdx = int.Parse(sn[1]);

                                selectedLn = new int[endIdx - startIdx + 1];
                                for (int k = startIdx; k <= endIdx; k++)
                                    selectedLn[k - startIdx] = k;

                                allLines = false;
                            }
                            else
                            {
                                sn = key_val[1].Split(null_delimiter, StringSplitOptions.RemoveEmptyEntries);

                                selectedLn = new int[sn.Length];
                                for (int k = 0; k < sn.Length; k++)
                                {
                                    selectedLn[k] = int.Parse(sn[k]);
                                }

                                allLines = false;
                            }

                            break;

                        case ADDTOLINESPARAMS.SET_PROPERTY:
                            SteelSection sec = sections[int.Parse(key_val[1])];
                            if (allLines)
                                foreach (BeamProperties bp in beamprops)
                                    bp.setSection(sec);
                            else
                                for (int k = 0; k < selectedLn.Length; k++)
                                    beamprops[selectedLn[k]].setSection(sec);
                            break;

                        case ADDTOLINESPARAMS.SET_THETA:
                            double theta = Double.Parse(key_val[1]);
                            if (allLines)
                                foreach (BeamProperties bp in beamprops)
                                    bp.setTheta(theta);
                            else
                                for (int k = 0; k < selectedLn.Length; k++)
                                    beamprops[selectedLn[k]].setTheta(theta);
                            break;
                    }
                }

                return true;

            }catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }


        }

        private void printInput()
        {
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            ed.WriteMessage("\n\nCount sections: " + sections.Count);
            foreach (SteelSection ssec in sections)
            {
                ed.WriteMessage(ssec.ToString());
            }

            ed.WriteMessage("\n\nCount points: " + points.Count);
            foreach (Point3d pt in points)
            {
                ed.WriteMessage(String.Format("\n{0} {1} {2}", pt.x, pt.y, pt.z));
            }

            ed.WriteMessage("\n\nCount lines: " + beamprops.Count);
            foreach (BeamProperties bp in beamprops)
            {
                ed.WriteMessage(bp.ToString());
            }
        }
    }
    
    public static class Extensions
    {
        public static T ToEnum<T>(this string value)
        {
            return (T)Enum.Parse(typeof(T), value, true);
        }
        
    }
}
